/*
 * Paraselene
 * Copyright (c) 2009  Akira Terasaki
 * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
 * 利用可能です。
 */
package paraselene.supervisor;

import paraselene.*;

/**
 * 遷移先指定。input を処理したページ(自分自身)に画面遷移します。<br>
 * ページのgetAjaxSupportが有効な場合、画面遷移が発生せず画面の再描画になります。
 */
public class Feedback extends Forward implements AjaxForward {
	private static final long serialVersionUID = 2L;
	private Page target;

	/**
	 * コンストラクタ。
	 * input と同じインスタンスで output を処理します。<br>
	 * セッションは維持されます。
	 */
	public Feedback() {
		this( SandBox.getCurrentPage() );
	}

	/**
	 * コンストラクタ。
	 * 指定ページの更新を要求します。そのページの output を処理します。<br>
	 * セッションは維持されます。
	 * @param page 更新対象。
	 */
	public Feedback( Page page ) {
		super( page.getID() );
		target = page;
	}

	Page getPage( Page from ) {
		return target;
	}

	/**
	 * 同じ動作であるか？
	 * Feedbackであれば同じ動作と見なします。
	 * @param o 比較対象。
	 * @return true:同一動作、false:異なる動作。
	 */
	public boolean equals( Object o ) {
		return o instanceof Feedback;
	}

	boolean isSamePage( Closure c ) {
		return ( c.close_page == target );
	}
}

